//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef INST_DATA_SECTION_HXX
# define INST_DATA_SECTION_HXX

# include "inst/data_section.hh"

# include "misc/contract.hh"

namespace inst
{

  //
  inline
  DataSection::DataSection(const std::string &name):
    Section(name)
  {
  }

  //
  inline int
  DataSection::size() const
  {
    return bytes_.size();
  }

  //
  inline void
  DataSection::add_space(unsigned space_size)
  {
    for (unsigned i = 0; i < space_size; ++i)
      bytes_.push_back(0);
  }

  inline void
  DataSection::add_byte(int b)
  {
    // FIXME: check b in byte range
    bytes_.push_back(b);
  }

  inline void
  DataSection::add_word(int w)
  {
    unsigned pow = 256 * 256 * 256;
    for (int i = 0; i < 4; ++i, pow /= 256)
      add_byte(w / pow);
  }

  //
  inline int
  DataSection::load_byte(unsigned offset) const
  {
    precondition(offset < bytes_.size());
    return (int8_t) bytes_[offset];
  }

} // namespace inst

#endif // !INST_DATA_SECTION_HXX
