//
// contract.hh: replacement for cassert
// Copyright (C) 2004 Benoit Perrot <benoit@lrde.epita.fr>
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// It is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//

/** \file misc/contract.hh
    \brief Replacement for cassert.

    The implementation is inspired from LRDE's contract.hh/contract.cc
    provided in the Tiger Compiler and the Vaucanson project.
*/

#ifndef MISC_CONTRACT_HH
# define MISC_CONTRACT_HH

# ifdef NDEBUG

#  define assertion(expr)         ((void) 0)
#  define invariant(expr)         ((void) 0)
#  define precondition(expr)      ((void) 0)
#  define postcondition(expr)     ((void) 0)

# else // NDEBUG

#  include <iostream>
#  include <cstdlib>

inline void __FailedCondition(const char* condType,
                              const char* condText,
                              const char* fileName,
                              int fileLine)
{
  std::cerr << fileName << ':'
            << fileLine
            << ": "
            << condType << " `"
            << condText << "' failed." << std::endl;
  abort();
}

#  define __TestCondition(condType,expr)                                \
  ((void) ((expr) ? 0 : (__FailedCondition( #condType, #expr,           \
                                           __FILE__, __LINE__ ), 0)))

#  define assertion(expr)         __TestCondition(Assertion,expr)
#  define invariant(expr)         __TestCondition(Invariant,expr)
#  define precondition(expr)      __TestCondition(Precondition,expr)
#  define postcondition(expr)     __TestCondition(Postcondition,expr)

# endif // !NDEBUG

#endif // !MISC_CONTRACT_HH
