//
// has.hh: finding element or key in STL containers.
// Copyright (C) 2003, 2004 Akim Demaille <akim@epita.fr> and
//                          Benoit Perrot <benoit@lrde.epita.fr>
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// It is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef MISC_HAS_HH
# define MISC_HAS_HH

# include <algorithm>

template <class Container>
inline typename Container::const_iterator
find (const Container &c, const typename Container::value_type &v)
{
  return std::find (c.begin (), c.end (), v);
}

template <class Container>
inline typename Container::iterator
find (Container &c, const typename Container::value_type &v)
{
  return std::find (c.begin (), c.end (), v);
}

template <class Container>
inline bool
has (const Container &c, const typename Container::value_type &v)
{
  // We specify the instance to solve a conflict between the
  // two finds above, that compete against each other because
  // the parameter Container can embed a "const".
  return find<Container> (c, v) != c.end ();
}

template <class Associative>
inline bool
has_key (const Associative &a, const typename Associative::key_type &k)
{
  // We specify the instance to solve a conflict between the
  // two finds above, that compete against each other because
  // the parameter Container can embed a "const".
  return a.find(k) != a.end ();
}

#endif // !MISC_HAS_HH
