//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef TASK_TASK_HH
# define TASK_TASK_HH

# include <string>
# include <list>

namespace task
{

  class Task
  {
  public:
    Task(const std::string &option, const std::string &module_name,
         const std::string &desc, const std::string &deps = "",
         bool needs_value = false);
    virtual ~Task();

  public:
    virtual void execute() const = 0;

  public:
    const std::string &long_opt() const;
    const std::string &short_opt() const;
    const std::string &module() const;
    const std::string &description(void) const;

    typedef std::list<std::string> deps_type;

    const deps_type &dependencies(void) const;

    bool needs_value() const;
    virtual bool set_value(const std::string &value) const;

  protected:
    std::string long_opt_;
    std::string short_opt_;
    std::string module_;
    std::string description_;
    deps_type dependencies_;

    bool needs_value_;
  };

} // namespace task

# include "task/task.hxx"

#endif// !TASK_TASK_HH
